# pygico_imagegen/config.py
from pydantic_settings import BaseSettings, SettingsConfigDict

class Settings(BaseSettings):
    BLOG_PATH: str = './blog.txt'
    SERVER_ADDRESS: str = '127.0.0.1:8188'
    GIMP_PATH: str = 'gimp-console-2.10.exe'
    WORKFLOW_PATH: str = './workflow.json'
    OUTPUT_DIR: str = './output'
    GEMINI_API_KEY: str

    model_config = SettingsConfigDict(env_file='.env', env_file_encoding='utf-8')

settings = Settings()



import os
from typing import Dict, Any
from loguru import logger
from dotenv import load_dotenv
from litellm import completion
import random
from art import text2art

class PromptGenerator:
    def __init__(self):
        load_dotenv()
        # self.api_key = os.getenv('GEMINI_API_KEY')
        # if not self.api_key:
        #     raise ValueError("GEMINI_API_KEYが設定されていません。")

    def generate(self, content: str) -> str:
        print(text2art(">> PromptGenerator"))
        """与えられたコンテンツを使用してプロンプトを生成する"""
        logger.info(f"コンテンツ \n'{content}'\n からプロンプトを生成します")
        prompt = f"下記のブログ記事の内容を表現するシンプルな画像の英語プロンプトを提案してください。英語プロンプトのみを出力してください：\n\n{content}"
        # os.environ['GEMINI_API_KEY'] = self.api_key
        response = completion(
            model="gpt-4o-mini", 
            messages=[{"role": "user", "content": prompt}]
        )
        generated_prompt = response.get('choices', [{}])[0].get('message', {}).get('content', '')
        logger.success(f"プロンプトを生成しました: \n{generated_prompt}")
        return generated_prompt

    def customize_workflow(self, workflow: Dict[str, Any], content: str) -> Dict[str, Any]:
        """ワークフローをカスタマイズし、プロンプトとシードを設定する"""
        logger.info("ワークフローをカスタマイズします")
        prompt = self.generate(content)
        workflow["283"]["inputs"]["text"] = f"""
        3D low poly model of [{prompt}], 
        isometric view, 
        2D game art style, 
        light background, 
        soft lighting, 
        low detail, white background, Intricate details and realistic textures, ((image center is blank area))
        """
        workflow["271"]["inputs"]["seed"] = random.randint(1, 1000000)
        logger.success("ワークフローのカスタマイズが完了しました")
        return workflow



import os
import json
from dotenv import load_dotenv
from loguru import logger

class ConfigManager:
    def __init__(self):
        logger.info("設定マネージャーを初期化しています")
        self.config = {}
        self._load_env()
        self._load_config_file()
        self._set_defaults()

    def _load_env(self):
        """環境変数を読み込みます。"""
        logger.info("環境変数を読み込んでいます")
        load_dotenv()
        self.config['BLOG_PATH'] = os.getenv('BLOG_PATH')
        self.config['SERVER_ADDRESS'] = os.getenv('SERVER_ADDRESS')
        self.config['GIMP_PATH'] = os.getenv('GIMP_PATH')
        self.config['WORKFLOW_PATH'] = os.getenv('WORKFLOW_PATH')

    def _load_config_file(self):
        """コンフィグファイルを読み込みます。"""
        config_path = os.path.expanduser("~/.config/pygico-imagegen/config.json")
        logger.info(f"コンフィグファイルを読み込んでいます: {config_path}")
        if os.path.exists(config_path):
            with open(config_path, 'r') as f:
                file_config = json.load(f)
                self.config.update(file_config)
        else:
            logger.warning(f"コンフィグファイルが見つかりません: {config_path}")

    def _set_defaults(self):
        """デフォルト値を設定します。"""
        logger.info("デフォルト設定を適用しています")
        defaults = {
            'BLOG_PATH': './blog.txt',
            'SERVER_ADDRESS': '127.0.0.1:8188',
            'GIMP_PATH': 'gimp',
            'WORKFLOW_PATH': './workflow.json',
            'OUTPUT_DIR': './output'
        }
        for key, value in defaults.items():
            if key not in self.config or self.config[key] is None:
                self.config[key] = value

    def get(self, key):
        """指定されたキーの設定値を取得します。"""
        value = self.config.get(key)
        if value is None:
            logger.warning(f"設定 '{key}' が見つかりません")
        return value

    def get_workflow(self):
        """ワークフローファイルを読み込みます。"""
        workflow_path = self.get('WORKFLOW_PATH')
        logger.info(f"ワークフローファイルを読み込んでいます: {workflow_path}")
        with open(workflow_path, 'r') as file:
            return json.load(file)

    def set(self, key, value):
        """指定されたキーに設定値を設定します。"""
        self.config[key] = value
        logger.info(f"設定 '{key}' を '{value}' に更新しました")



# pygico_imagegen/cli.py
import argparse
from loguru import logger
from .content_analysis.blog_extractor import BlogExtractor
from .content_analysis.prompt_generator import PromptGenerator
from .image_generation.comfy_interface import ComfyInterface
from .image_processing.gimp_executor import GimpExecutor
from .utils.config_manager import ConfigManager
from .config import settings

def main():
    parser = argparse.ArgumentParser(description="PyGiCo-ImageGen: ブログコンテンツから画像を生成し処理します。")
    parser.add_argument("--blog", help="ブログファイルのパス", default=settings.BLOG_PATH)
    parser.add_argument("--output", help="出力画像のパス", required=True)
    parser.add_argument("--text", help="画像に追加するテキスト", default=None)
    args = parser.parse_args()

    config = ConfigManager()
    logger.info("PyGiCo-ImageGen を開始します")

    try:
        # ブログ内容の抽出
        logger.info(f"ブログファイル {args.blog} から内容を抽出します")
        extractor = BlogExtractor(args.blog)
        blog_content = extractor.extract()

        # プロンプトの生成とワークフローのカスタマイズ
        logger.info("画像生成プロンプトを作成し、ワークフローをカスタマイズします")
        generator = PromptGenerator()
        workflow = config.get_workflow()
        customized_workflow = generator.customize_workflow(workflow, blog_content)

        # 画像の生成
        logger.info("ComfyUI を使用して画像を生成します")
        comfy = ComfyInterface(settings.SERVER_ADDRESS)
        image = comfy.generate_image(customized_workflow)

        # テキストの追加（オプション）
        if args.text:
            logger.info(f"画像にテキスト '{args.text}' を追加します")
            gimp = GimpExecutor(settings.GIMP_PATH)
            temp_path = f"{args.output}.temp.png"
            image[-1].save(temp_path)
            gimp.add_text(temp_path, args.text, args.output)
        else:
            logger.info(f"生成された画像を {args.output} に保存します")
            image[-1].save(args.output)

        logger.success(f"処理が完了しました。結果は {args.output} に保存されています")

    except Exception as e:
        logger.error(f"処理中にエラーが発生しました: {str(e)}")
        raise

if __name__ == "__main__":
    main()



import json
from loguru import logger
from art import text2art
from pygimp.pygimp_core import PyGIMP

class GimpExecutor:
    def __init__(self, gimp_path='gimp-console-2.10.exe'):
        self.pygimp = PyGIMP(gimp_path)

    def add_text(self, input_image, text, output_path, config_path="gimp_script_config.json"):
        # ASCIIアートのタイトルを表示
        print(text2art(">> PyGIMP"))

        # 設定を読み込み
        with open(config_path, 'r', encoding='utf-8') as f:
            config = json.load(f)

        # configが辞書であることを確認
        if not isinstance(config, dict):
            raise ValueError("読み込まれた設定が辞書ではありません")

        # 提供された引数でconfigを更新
        config["input_image"] = input_image
        config["output_path"] = output_path
        config["text"] = text

        # PyGIMPの設定を更新
        self.pygimp.update_config(**config)

        # GIMPスクリプトを実行
        self.pygimp.execute_script()

        logger.info(f"GIMPスクリプトが実行されました。出力: {output_path}")



import os
from loguru import logger
from art import text2art

class BlogExtractor:
    def __init__(self, blog_path):
        self.blog_path = blog_path

    def extract(self, max_length=300):
        """ブログファイルからコンテンツを抽出します。"""
        print(text2art(">> BlogExtractor"))
        logger.info(f"{self.blog_path} からブログコンテンツを抽出中...")

        if not os.path.exists(self.blog_path):
            logger.error(f"ブログファイルが見つかりません: {self.blog_path}")
            raise FileNotFoundError(f"ブログファイルが見つかりません: {self.blog_path}")

        try:
            with open(self.blog_path, 'r', encoding='utf-8') as file:
                content = file.read(max_length)
            
            logger.success(f"ブログコンテンツの抽出が完了しました（{len(content)}文字）")
            return content
        except Exception as e:
            logger.error(f"ブログコンテンツの抽出中にエラーが発生しました: {str(e)}")
            raise



"""PyGiCo-ImageGen: Python, GIMP, and ComfyUI based image generation and text addition tool."""

__version__ = "0.1.0"

from .content_analysis.blog_extractor import BlogExtractor
from .content_analysis.prompt_generator import PromptGenerator
from .image_generation.comfy_interface import ComfyInterface
from .image_processing.gimp_executor import GimpExecutor
from .utils.config_manager import ConfigManager

__all__ = [
    "BlogExtractor",
    "PromptGenerator",
    "ComfyInterface",
    "GimpExecutor",
    "ConfigManager",
]



"""BlogExtractor のユニットテスト"""

import pytest
import os
from pygico_imagegen.content_analysis.blog_extractor import BlogExtractor

@pytest.fixture
def sample_blog_file(tmp_path):
    blog_content = """サンプルブログのタイトル
これはサンプルブログの内容です。
複数行にわたるコンテンツを含んでいます。
テスト用のデータとして使用します。
"""
    blog_file = tmp_path / "sample_blog.txt"
    blog_file.write_text(blog_content, encoding="utf-8")
    return str(blog_file)

def test_extract_content(sample_blog_file):
    extractor = BlogExtractor(sample_blog_file)
    content = extractor.extract()
    assert content.startswith("サンプルブログのタイトル")
    assert "複数行にわたるコンテンツ" in content

def test_extract_with_max_length(sample_blog_file):
    extractor = BlogExtractor(sample_blog_file)
    content = extractor.extract(max_length=20)
    assert len(content) == 20

def test_file_not_found():
    with pytest.raises(FileNotFoundError):
        extractor = BlogExtractor("non_existent_file.txt")
        extractor.extract()

def test_empty_file(tmp_path):
    empty_file = tmp_path / "empty.txt"
    empty_file.write_text("", encoding="utf-8")
    extractor = BlogExtractor(str(empty_file))
    content = extractor.extract()
    assert content == ""

def test_long_content(tmp_path):
    long_content = "A" * 1000
    long_file = tmp_path / "long_content.txt"
    long_file.write_text(long_content, encoding="utf-8")
    extractor = BlogExtractor(str(long_file))
    content = extractor.extract()
    assert len(content) == 300  # デフォルトの最大長が300文字であることを確認

def test_custom_max_length(sample_blog_file):
    extractor = BlogExtractor(sample_blog_file)
    content = extractor.extract(max_length=50)
    assert len(content) <= 50

def test_utf8_encoding(tmp_path):
    utf8_content = "こんにちは、世界！"
    utf8_file = tmp_path / "utf8_content.txt"
    utf8_file.write_text(utf8_content, encoding="utf-8")
    extractor = BlogExtractor(str(utf8_file))
    content = extractor.extract()
    assert content == utf8_content



import json
import requests
from loguru import logger
from PIL import Image
import io
import time
import os
from urllib import request, parse
from art import text2art

class ComfyInterface:
    def __init__(self, server_address="127.0.0.1:8188"):
        self.server_address = server_address
        self.base_url = f"http://{server_address}"
        logger.info(f"ComfyUIインターフェースを初期化しました。サーバーアドレス: {server_address}")

    def generate_image(self, workflow):
        print(text2art(">> ComfyInterface"))
        """ComfyUIを使用して画像を生成します。"""
        logger.info("ComfyUIを使用した画像生成プロセスを開始します")

        try:
            # 画像生成のキュー登録
            prompt_id = self._queue_prompt(workflow)
            
            # 生成完了を待機
            outputs = self._wait_for_generation(prompt_id)
            
            if not outputs:
                logger.warning("画像生成プロセスは完了しましたが、出力が見つかりません")
                return []

            # 画像の取得と返却
            images = []
            for node_output in outputs.values():
                for image_data in node_output.get('images', []):
                    images.append(self._get_image(image_data))
            
            logger.success(f"{len(images)}枚の画像の生成が完了しました")
            return images

        except Exception as e:
            logger.error(f"画像生成中にエラーが発生しました: {str(e)}")
            raise

    def _queue_prompt(self, workflow):
        """画像生成をキューに登録します。"""
        logger.info("画像生成をComfyUIのキューに登録しています")

        p = {"prompt": workflow}
        data = json.dumps(p).encode('utf-8')
        endpoint = f"{self.base_url}/prompt"
        logger.info(f"endpoint: {endpoint}")
        
        headers = {'Content-Type': 'application/json'}
        try:
            response = requests.post(endpoint, data=data, headers=headers)
            response.raise_for_status()  # HTTPエラーがあれば例外を発生させる
            response_json = response.json()
            logger.info(f"サーバーレスポンス: {response_json}")
            return response_json['prompt_id']
        except requests.RequestException as e:
            logger.error(f"リクエスト中にエラーが発生しました: {str(e)}")
            raise


    def _wait_for_generation(self, prompt_id):
        """画像生成の完了を待機し、生成された画像の情報を返します。"""
        logger.info(f"画像生成の完了を待機しています。プロンプトID: {prompt_id}")
        max_attempts = 60  # 最大待機時間（秒）
        debug_folder = "debug_logs"
        os.makedirs(debug_folder, exist_ok=True)

        for attempt in range(max_attempts):
            try:
                response = requests.get(f"{self.base_url}/history/{prompt_id}")
                response.raise_for_status()
                data = response.json()

                # デバッグ用にJSONレスポンスを保存
                debug_file = os.path.join(debug_folder, f"response_{prompt_id}_{attempt}.json")
                with open(debug_file, 'w', encoding='utf-8') as f:
                    json.dump(data, f, ensure_ascii=False, indent=2)
                logger.debug(f"レスポンスをデバッグファイルに保存しました: {debug_file}")

                if prompt_id in data:
                    prompt_data = data[prompt_id]
                    if "outputs" in prompt_data:
                        outputs = prompt_data["outputs"]
                        # 出力が存在し、少なくとも1つのノードに画像がある場合
                        if any('images' in node_output for node_output in outputs.values()):
                            logger.success("画像生成が完了しました")
                            return outputs
                    elif "executing" in prompt_data and not prompt_data["executing"]:
                        logger.warning("生成プロセスが完了しましたが、出力が見つかりません")
                        return None

                logger.info(f"生成進行中... 試行回数: {attempt + 1}")
            except requests.RequestException as e:
                logger.warning(f"履歴の取得中にエラーが発生しました: {str(e)}")
            except json.JSONDecodeError as e:
                logger.error(f"JSONのデコードに失敗しました: {str(e)}")
                # JSONデコードエラーの場合、生のレスポンス内容をファイルに保存
                error_file = os.path.join(debug_folder, f"error_response_{prompt_id}_{attempt}.txt")
                with open(error_file, 'w', encoding='utf-8') as f:
                    f.write(response.text)
                logger.debug(f"エラーレスポンスを保存しました: {error_file}")
            except Exception as e:
                logger.error(f"予期せぬエラーが発生しました: {str(e)}")
            
            time.sleep(5)
        
        logger.error("画像生成がタイムアウトしました")
        raise TimeoutError("画像生成がタイムアウトしました")
    
    def _get_image(self, image_data):
        """生成された画像を取得します。"""
        logger.info("生成された画像を取得しています")
        try:
            # image_dataから直接ファイル名を取得
            filename = image_data.get('filename', '')
            subfolder = image_data.get('subfolder', '')
            image_type = image_data.get('type', 'output')  # デフォルトは'output'とする

            if not filename:
                raise ValueError("画像ファイル名が見つかりません")

            url = f"{self.base_url}/view?filename={filename}&subfolder={subfolder}&type={image_type}"
            logger.debug(f"画像取得URL: {url}")

            response = requests.get(url)
            response.raise_for_status()
            image = Image.open(io.BytesIO(response.content))
            logger.success(f"画像の取得が完了しました: {filename}")
            return image
        except requests.RequestException as e:
            logger.error(f"画像の取得中にエラーが発生しました: {str(e)}")
            raise
        except Exception as e:
            logger.error(f"画像の処理中に予期せぬエラーが発生しました: {str(e)}")
            raise



from setuptools import setup

setup(
    name="pygico-imagegen",
    version="0.1.0",
    packages=["pygico_imagegen"],
    package_data={
        "pygico_imagegen": ["py.typed"],
    },
    include_package_data=True,
)
